'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var deepClone = _interopDefault(require('clone-deep'));

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

function assertNever(value, key) {
  throw new Error("Item of key name " + key + " had a type that is unsupported by this: " + JSON.stringify(value));
}

/**
 * Returns an array of key/value pairs
 * @param object the object you want to translate into an array of key/value pairs
 */

var toKVP = function toKVP(object) {
  return Object.keys(object).map(function (key) {
    var keyStrict = key;
    return {
      key: key,
      value: object[keyStrict]
    };
  });
};

var determineNewValue = function determineNewValue(input) {
  var keyToOverride = input.keyToOverride,
      oldValue = input.oldValue,
      newValue = input.newValue; // Since the consumer supplied a newValue, it's time to figure out how to properly replace it

  if (typeof newValue === 'string' || typeof newValue === 'number' || typeof newValue === 'bigint' || typeof newValue === 'function' || typeof newValue === 'boolean' || typeof newValue === 'symbol') {
    // There's nothing to iterate over, so return the override
    return {
      keyToOverride: keyToOverride,
      newValue: newValue
    };
  } else if (typeof newValue === 'object') {
    if (Array.isArray(newValue)) {
      // replace the whole array
      return {
        keyToOverride: keyToOverride,
        newValue: newValue
      };
    }

    if (newValue === null) {
      return {
        keyToOverride: keyToOverride,
        newValue: newValue
      };
    }

    if (typeof oldValue !== 'object' || !oldValue) {
      return {
        keyToOverride: keyToOverride,
        newValue: newValue
      };
    } // eslint-disable-next-line @typescript-eslint/no-use-before-define


    return {
      keyToOverride: keyToOverride,
      newValue: mergePartiallyShallow(oldValue, newValue)
    };
  } else if (newValue === undefined) {
    // But if the consumer provided the property, they probably wanted to replace the value with undefined. And if we're wrong then TypeScript will have caught this.
    return {
      keyToOverride: keyToOverride,
      newValue: newValue
    };
  } else {
    /* istanbul ignore next */
    // ignore this line since it literally can't be hit
    throw assertNever(newValue, keyToOverride);
  }
};
/**
 * Returns a copy of seed with any non-undefined parameters from override
 * @param seed the object that is used establish the start of what you want the result to look like. This is the object that will be overriden before a result is produced
 * @param override the data that will be used when replacing the seed's key/values
 */


var mergePartiallyShallow = function mergePartiallyShallow(seed, override) {
  if (!seed) {
    throw new TypeError("seed must be provided and must not be null/undefined. It was " + typeof seed);
  } // TODO: we can stop relying on runtime type checks if/when negated types are introduced: https://github.com/microsoft/TypeScript/pull/29317#issuecomment-692750988


  if (Array.isArray(seed)) {
    throw new TypeError('this function only supports non-array objects.');
  }

  var seedCopy = deepClone(seed); // Short-circuit if override was not provided

  if (!override) {
    return seedCopy;
  } // Lie #1 - the object and it's override are objects with iterable keys. More information here: https://github.com/microsoft/TypeScript/issues/35859#issuecomment-687323281


  var seedRecord = seedCopy;
  var overrideRecord = override;
  var overrideKeyValuePairs = toKVP(overrideRecord);

  for (var _iterator = _createForOfIteratorHelperLoose(overrideKeyValuePairs), _step; !(_step = _iterator()).done;) {
    var overrideKVP = _step.value;
    var keyToOverride = overrideKVP.key;

    var _determineNewValue = determineNewValue({
      keyToOverride: keyToOverride,
      oldValue: seedRecord[keyToOverride],
      newValue: overrideKVP.value
    }),
        newValue = _determineNewValue.newValue;

    seedRecord[keyToOverride] = newValue;
  }

  return seedCopy;
};
/**
 * Returns a copy of seed with any non-undefined parameters from override
 * @param seed the object that is used establish the start of what you want the result to look like. This is the object that will be overriden before a result is produced
 * @param override the data that will be used when replacing the seed's key/values
 */


var mergePartiallyDeep = function mergePartiallyDeep(seed, override) {
  return mergePartiallyShallow(seed, override);
};

var mergePartially = {
  shallow: mergePartiallyShallow,
  deep: mergePartiallyDeep
};

exports.mergePartially = mergePartially;
//# sourceMappingURL=merge-partially.cjs.development.js.map
